#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#
#  Copyright(c)  2002  Richard B. Johnson
#
#  This program may be distributed under the GNU Public License
#  version 2, as published by the Free Software Foundation, Inc.,
#  59 Temple Place, Suite 330 Boston, MA, 02111.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#
#  File lowlevel.S             Created 3-NOV-2002     Richard B. Johnson
#
#       Most of the procedures in this file could easily be performed
#       using the 'C' language. However, the resulting code is quite
#       large. Therefore, once I find a 'C' procedure that is using
#       a lot of space, I make a short-and-sweet assembly-language
#       procedure to replace it, if it's easily coded.
#
.section        .text

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Intel specific:
#       char *strstr(const char *haystack, const char *needle);
#
#    Returns a pointer to the substring in haystack if it is found.
#    Returns NULL if it is not found.
#
#
STR = 0x18              # Offset of the string
SUB = 0x1c              # Offset of the substring
.global strstr
.type   strstr,@function
.align  0x04
strstr: pushl   %ebx
        pushl   %ecx
        pushl   %edx
        pushl   %esi
        pushl   %edi
        cld
        movl    SUB(%esp), %ebx # Get address of the substring
        movl    STR(%esp), %esi # Get address of the string
        pushl   %ebx            # Put in on the stack
        call    strlen          # Get length of string in EAX
        addl    $0x04, %esp     # Level stack
        movl    %eax, %edx      # Save substring length
        pushl   %esi            # Address of the string
        call    strlen          # Get its length
        addl    $0x04, %esp     # Level stack
        movl    %eax, %ecx      # Set string length
        addl    %esi, %ecx      # ECX = last buffer location
        subl    %edx, %ecx      # Max address to scan
        xorl    %eax, %eax      # Possible NULL return
1:      pushl   %esi            # Address of the string
        pushl   %ecx            # String byte-count
        movl    %ebx, %edi      # Address of the substring
        movl    %edx, %ecx      # Length to scan
        repz    cmpsb           # Compare
        popl    %ecx            # String byte-cout
        popl    %esi            # Address of the string
        jz      2f              # Substring was found
        incl    %esi            # Try next offset
        cmpl    %ecx, %esi      # check limits
        jbe     1b              # Continue
        jmp     3f              # All done
2:      movl    %esi, %eax      # Substring location
3:      popl    %edi
        popl    %esi
        popl    %edx
        popl    %ecx
        popl    %ebx
        ret
.size   strstr,.-strstr
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel Specific string-length function. This gets to be executed
#  if our shared library is linked first.
#
#       Stack offsets for input variables to functions.
#
INPSTR = 0x0c
INPLEN = 0x10
#
.global strnlen
.type strnlen,@function
.align  0x04
strnlen:
        pushl   %ecx
        pushl   %edi
        movl    INPSTR(%esp), %edi
        movl    INPLEN(%esp), %ecx
        xorl    %eax, %eax              # Cheap zero
        cld
        repnz   scasb                   # Continue until CX=0 or the null
        movl    %edi, %eax              # Get one past the null
        subl    INPSTR(%esp), %eax      # Starting address
        subl    $1, %eax                # String length
        adcl    $0, %eax                # Not below zero
        popl    %edi
        popl    %ecx
        ret
.size   strnlen,.-strnlen
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel Specific string-length function. This gets to be executed
#  if our shared library is linked first.
#
#       Stack offsets for input variables to functions.
#
INPSTR = 0x0c
#
.global strlen
.type strlen,@function
.align  0x04
strlen: pushl   %ecx
        pushl   %edi
        movl    INPSTR(%esp), %edi
        xorl    %ecx, %ecx      # Cheap zero
        movl    %ecx, %eax      # Clone
        decl    %ecx            # Cleap -1
        cld
        repnz   scasb
        movl    %ecx, %eax
        notl    %eax
        decl    %eax
        popl    %edi
        popl    %ecx
        ret
.size   strlen,.-strlen
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel specific strcmp.
#
DST = 0x10
SRC = 0x14
.global strcmp
.type strcmp,@function
.align  0x04
strcmp: pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    SRC(%esp), %edi
        xorl    %ecx, %ecx      # Cheap zero
        movl    %ecx, %eax      # Clone
        decl    %ecx            # Cleap -1
        cld
        repnz   scasb
        notl    %ecx
        decl    %ecx            # ECX = string length
        movl    SRC(%esp), %edi
        movl    DST(%esp), %esi
        repz    cmpsb
        jz      1f              # Strings are equal
        sbbl    $0, %eax        # SRC is less than DEST
        js      1f              # Done
        addl    $1, %eax        # Make + 1
1:      popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   strcmp,.-strcmp
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel specific strncmp.
#
DST = 0x10
SRC = 0x14
LEN = 0x18
.global strncmp
.type strncmp,@function
.align  0x04
strncmp:
        pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    SRC(%esp), %edi # Point to string
        movl    LEN(%esp), %ecx # Length to scan
        movl    %edi, %esi      # Save start
        xorl    %eax, %eax      # Cheap zero
        cld
        repnz   scasb
        movl    %edi, %ecx      # End of string
        subl    %esi, %ecx      # Compare length
        movl    SRC(%esp), %edi
        movl    DST(%esp), %esi
        repz    cmpsb
        jz      1f              # Strings are equal
        sbbl    $0, %eax        # SRC is less than DEST
        js      1f              # Done
        addl    $1, %eax        # Make + 1
1:      popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   strncmp,.-strncmp
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel-specifc char *strrchr(const char *str, int char)
#  Returns a pointer to the LAST such character in an input
#  string. Returns NULL if no such character exists. If the
#  character is a 0x00, then return with a pointer to it too.
#
#       Stack offsets for input variables to functions.
#
CHRFND = 0x14
INPSTR = 0x10
.global strrchr
.type strrchr,@function
.align  0x04
strrchr:
        pushf
        pushl   %ecx
        pushl   %edi
        movl    INPSTR(%esp), %edi
        push    %edi                    # Pointer to the string
        call    strlen                  # Get string length
        addl    $0x04, %esp             # Level stack
        movl    %eax, %ecx              # Max scan length
        addl    %ecx, %edi              # Set on the NULL
        movl    CHRFND(%esp), %eax      # Character to find
        std                             # Backwards
        repnz   scasb                   # Try to find it
        movl    %ecx, %eax              # Assume it's not found
        jnz     1f                      # Not found
        incl    %edi                    # It stops one-past the point
        movl    %edi, %eax              # Return pointer
1:      popl    %edi
        popl    %ecx
        popf
        ret
.size   strrchr,.-strrchr
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel-specifc char *strchr(const char *str, int char)
#  Returns a pointer to the first such character in an input
#  string. Returns NULL if no such character exists. If the
#  character is a 0x00, then return with a pointer to it too.
#
#       Stack offsets for input variables to functions.
#
CHRFND = 0x10
INPSTR = 0x0c
.global strchr
.type strchr,@function
.align  0x04
strchr: pushl   %ecx
        pushl   %edi
        movl    INPSTR(%esp), %edi
        push    %edi                    # Pointer to the string
        call    strlen                  # Get string length
        addl    $0x04, %esp             # Level stack
        movl    %eax, %ecx              # Max scan length
        incl    %ecx                    # Include null also
        movl    CHRFND(%esp), %eax      # Character to find
        cld
        repnz   scasb                   # Try to find it
        movl    $0, %eax                # Assume it's not found
        jnz     1f                      # Not found
        decl    %edi                    # It stops one-past the point
        movl    %edi, %eax              # Return pointer
1:      popl    %edi
        popl    %ecx
        ret
.size   strchr,.-strchr
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel-specifc char *strnchr(const char *str, int char, size_t len)
#  Returns a pointer to the first such character in an input
#  string. Returns NULL if no such character exists. If the
#  character is a 0x00, then return with a pointer to it too.
#
#       Stack offsets for input variables to functions.
#
SCNLEN = 0x14
CHRFND = 0x10
INPSTR = 0x0c
.global strnchr
.type strnchr,@function
.align  0x04
strnchr: pushl  %ecx
        pushl   %edi
        movl    INPSTR(%esp), %edi
        push    %edi                    # Pointer to the string
        call    strlen                  # Get string length
        addl    $0x04, %esp             # Level stack
        movl    %eax, %ecx              # Max scan length
        incl    %ecx                    # Include null also
        movl    SCNLEN(%esp), %eax      # Get input scan length
        cmpl    %eax, %ecx              # See if greater or equal.
        jc      1f                      # Greater than string length
        movl    %eax, %ecx              # Substitute input length
1:      movl    CHRFND(%esp), %eax      # Character to find
        cld
        repnz   scasb                   # Try to find it
        movl    $0, %eax                # Assume it's not found
        jnz     2f                      # Not found
        decl    %edi                    # It stops one-past the point
        movl    %edi, %eax              # Return pointer
2:      popl    %edi
        popl    %ecx
        ret
.size   strnchr,.-strnchr
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel-specifc char *memchr(const char *str, int char, int len)
#  Returns a pointer to the first such character in an input
#  string. Returns NULL if no such character exists. If the
#  character is a 0x00, then return with a pointer to it too.
#
#       Stack offsets for input variables to functions.
#
STRLEN = 0x14
CHRFND = 0x10
INPSTR = 0x0c
.global memchr
.type memchr,@function
.align  0x04
memchr: pushl   %ecx
        pushl   %edi
        movl    INPSTR(%esp), %edi
        movl    CHRFND(%esp), %eax      # Character to find
        movl    STRLEN(%esp), %ecx      # Length
        cld
        repnz   scasb                   # Try to find it
        movl    $0, %eax                # Assume it's not found
        jnz     1f                      # Not found
        decl    %edi                    # It stops one-past the point
        movl    %edi, %eax              # Return pointer
1:      popl    %edi
        popl    %ecx
        ret
.size   memchr,.-memchr
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Intel-specifc char *rawmemchr(const char *str, int char)
#  Returns a pointer to the first such character in an input
#  string. Returns NULL if no such character exists. If the
#  character is a 0x00, then return with a pointer to it too.
#
#       Stack offsets for input variables to functions.
#
CHRFND = 0x10
INPSTR = 0x0c
.global rawmemchr
.type rawmemchr,@function
.global __rawmemchr
.type __rawmemchr,@function
.align  0x04
rawmemchr:
__rawmemchr:
        pushl   %ecx
        pushl   %edi
        movl    INPSTR(%esp), %edi
        movl    CHRFND(%esp), %eax      # Character to find
        movl    $-1, %ecx               # Length
        cld
        repnz   scasb                   # Try to find it
        movl    $0, %eax                # Assume it's not found
        jnz     1f                      # Not found
        decl    %edi                    # It stops one-past the point
        movl    %edi, %eax              # Return pointer
1:      popl    %edi
        popl    %ecx
        ret
.size   __rawmemchr,.-__rawmemchr
.size   rawmemchr,.-rawmemchr
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Intel-specific char *strcpy(char *dest, const char *src);
#       This gets executed if our shared library gets linked first.
#
#       Stack offsets for input variables to functions.
#
OUTSTR = 0x10
INPSTR = 0x14
.global strcpy
.type strcpy,@function
.align  0x04
strcpy: pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    OUTSTR(%esp), %edi
        movl    INPSTR(%esp), %esi
        pushl   %esi                    # Input string on stack
        call    strlen                  # Get length
        addl    $0x04, %esp             # Level stack
        movl    %eax, %ecx              # Length in ECX
        incl    %ecx                    # Copy the null-byte also
        pushl   %edi                    # Destination
        cld                             # Forwards
        shr     $1,%ecx                 # Make words
        pushf                           # Save any extra bytes
        shr     $1,%ecx                 # Make longwords
        rep     movsl                   # Move any longwords
        adc     %ecx,%ecx               # Any extra word
        rep     movsw                   # Copy word
        popf                            # Remains from any bytes
        adc     %ecx,%ecx               # Any odd byte
        rep     movsb                   # Copy any odd byte
        popl    %eax                    # Destination return value
        popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   strcpy,.-strcpy
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Intel-specific char *strncpy(char *dest, const char *src, size_t len);
#       This gets executed if our shared library gets linked first.
#
#       This does __not__ pad the destination string with nulls!
#       Doing this is dumb and wastes time. It should not be the
#       behavior described by the `man` pages.
#
#       Stack offsets for input variables to functions.
#
OUTSTR = 0x14
INPSTR = 0x18
CPYLEN = 0x1c
#
.global strncpy
.type strncpy,@function
.align  0x04
strncpy:
        pushl   %ebx
        pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    OUTSTR(%esp), %edi
        movl    INPSTR(%esp), %esi
        movl    CPYLEN(%esp), %ebx      # Get maximum length allowed
        pushl   %edi                    # Destination (stays on the stack)
        pushl   %esi                    # Input string on stack
        call    strlen                  # Get length
        addl    $0x04, %esp             # Level stack
        movl    %eax, %ecx              # Length in ECX
        incl    %ecx                    # Copy the null-byte also
        cmpl    %ecx, %ebx              # Check limits
        jnc     1f                      # It's okay
        movl    %ebx, %ecx              # Substitute max allowed
1:      cld                             # Forwards
        shr     $1,%ecx                 # Make words
        pushf                           # Save any extra bytes
        shr     $1,%ecx                 # Make longwords
        rep     movsl                   # Move any longwords
        adc     %ecx,%ecx               # Any extra word
        rep     movsw                   # Copy word
        popf                            # Remains from any bytes
        adc     %ecx,%ecx               # Any odd byte
        rep     movsb                   # Copy any odd byte
        popl    %eax                    # Destination return value
        popl    %edi
        popl    %esi
        popl    %ecx
        popl    %ebx
        ret
.size   strncpy,.-strncpy
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Intel-specific char *strncat(char *dest, const char *src, size_t len);
#       This gets executed if our shared library gets linked first.
#
#       Stack offsets for input variables to functions.
#
OUTSTR = 0x10
INPSTR = 0x14
CPYLEN = 0x18
#
.global strncat
.type strncat,@function
.align  0x04
strncat:
        pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    OUTSTR(%esp), %edi
        movl    INPSTR(%esp), %esi
        movl    CPYLEN(%esp), %ecx      # Get maximum amount to copy
        pushl   %edi                    # Destination string on stack
        call    strlen                  # Get length of destrination
        addl    %eax, %edi              # Offset to start copy
        pushl   %esi                    # Input string on stack
        call    strlen                  # Get length
        addl    $0x04, %esp             # Level stack, EDI still on stack
        incl    %eax                    # Include null-byte
        cmpl    %eax, %ecx              # Check length
        jbe     1f                      # Use max length
        movl    %eax, %ecx              # Substitute count
1:      cld                             # Forwards
        shr     $1,%ecx                 # Make words
        pushf                           # Save any extra bytes
        shr     $1,%ecx                 # Make longwords
        rep     movsl                   # Move any longwords
        adc     %ecx,%ecx               # Any extra word
        rep     movsw                   # Copy word
        popf                            # Remains from any bytes
        adc     %ecx,%ecx               # Any odd byte
        rep     movsb                   # Copy any odd byte
        popl    %eax                    # Destination return value
        popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   strncat,.-strncat
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Intel-specific char *strcat(char *dest, const char *src);
#       This gets executed if our shared library gets linked first.
#
#       Stack offsets for input variables to functions.
#
OUTSTR = 0x10
INPSTR = 0x14
CPYLEN = 0x18
#
.global strcat
.type strcat,@function
.align  0x04
strcat: pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    OUTSTR(%esp), %edi
        movl    INPSTR(%esp), %esi
        pushl   %edi                    # Destination string on stack
        call    strlen                  # Get length of destrination
        addl    %eax, %edi              # Offset to start copy
        pushl   %esi                    # Input string on stack
        call    strlen                  # Get length
        addl    $0x04, %esp             # Level stack, EDI still on stack
        movl    %eax, %ecx              # Length in ECX
        incl    %ecx                    # Copy the null-byte also
        cld                             # Forwards
        shr     $1,%ecx                 # Make words
        pushf                           # Save any extra bytes
        shr     $1,%ecx                 # Make longwords
        rep     movsl                   # Move any longwords
        adc     %ecx,%ecx               # Any extra word
        rep     movsw                   # Copy word
        popf                            # Remains from any bytes
        adc     %ecx,%ecx               # Any odd byte
        rep     movsb                   # Copy any odd byte
        popl    %eax                    # Destination return value
        popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   strcat,.-strcat
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Intel-specific memory copy routine in which the source
#       and destination may overlap.
#
#       void *memmove(void *dst, void *src, size_t len);
#
#       Stack offsets for input variables to functions.
#
OUTSTR = 0x10
INPSTR = 0x14
CPYLEN = 0x18
#
.global memmove
.type memmove,@function
.align  0x04
memmove:
        pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    OUTSTR(%esp), %edi
        movl    INPSTR(%esp), %esi
        movl    CPYLEN(%esp), %ecx
        pushl   %edi                    # Destination
        cld                             # Assume forwards.
        cmpl    %esi, %edi              # Is destination LESS than source?
        jbe     1f                      # Yes, we can copy normally
        std                             # Direction flag backwards
        movl    %ecx, %eax              # Copy length
        decl    %eax                    # Convert to offset
        addl    %eax, %esi              # Start at the end of source
        addl    %eax, %edi              # Start at the end of destination
1:      rep     movsb                   # Copy everything as bytes
        cld                             # Normalize direction flag
        popl    %eax                    # Destination return value
        popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   memmove,.-memmove
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Intel-specific memory copy routine.
#
#       void *memcpy(void *dst, void *src, size_t len);
#       This is a callable procedure instead of the built-in
#       inline memcpy(). This makes the code shorter, but it
#       runs at the same speed. This gets executed if our
#       shared library gets linked first.
#
#       Stack offsets for input variables to functions.
#
OUTSTR = 0x10
INPSTR = 0x14
CPYLEN = 0x18
#
.global memcpy
.type memcpy,@function
.align  0x04
memcpy: pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    OUTSTR(%esp), %edi
        movl    INPSTR(%esp), %esi
        movl    CPYLEN(%esp), %ecx
        pushl   %edi                    # Destination
        cld                             # Forwards
        shr     $1,%ecx                 # Make words
        pushf                           # Save any extra bytes
        shr     $1,%ecx                 # Make longwords
        rep     movsl                   # Move any longwords
        adc     %ecx,%ecx               # Any extra word
        rep     movsw                   # Copy word
        popf                            # Remains from any bytes
        adc     %ecx,%ecx               # Any odd byte
        rep     movsb                   # Copy any odd byte
        popl    %eax                    # Destination return value
        popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   memcpy,.-memcpy
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Intel-specific memory copy routine.
#
#       void *memccpy(void *dst, void *src, int c, size_t len);
#       This is a callable procedure instead of the built-in
#       inline memcpy(). This makes the code shorter, but it
#       runs at the same speed. This gets executed if our
#       shared library gets linked first.
#
#       Stack offsets for input variables to functions.
#
OUTSTR = 0x10
INPSTR = 0x14
CHFIND = 0x18
CPYLEN = 0x1C
#
.global memccpy
.type memccpy,@function
.align  0x04
memccpy:
	pushl   %ecx
        pushl   %esi
        pushl   %edi
        movl    OUTSTR(%esp), %edi	# Destination
        movl    INPSTR(%esp), %esi	# Source
        movl    CPYLEN(%esp), %ecx	# Length
	movl	CHFIND(%esp), %eax	# Character to find
	cld				# Forwards
	movb	%al,%ah			# Save the character
1:	lodsb				# Get the character
	stosb				# Save in destination
	cmpb	%al,%ah			# Is it the end
	jnz	2f			# No
	movl	%edi, %eax		# Next character in destination
	jmp	3f			# Return witn pointer
2:	loop	1b			# Continue
	xorl	%eax,%eax		# Character never found
3:	popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   memccpy,.-memccpy
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This compares two strings. It returns 0 if they compare.
#       It returns non-zero if they don't. Unlike memcmp(), we
#       don't return a value based upon a greater or lesser string
#       value. This gets executed if our shared library gets loaded
#       first.
#
OUTSTR = 0x10
INPSTR = 0x14
CPYLEN = 0x18
#
.global memcmp
.type memcmp,@function
.align  0x04
memcmp: pushl   %ecx
        pushl   %esi
        pushl   %edi
        xorl    %eax, %eax              # Assume a good compare
        movl    OUTSTR(%esp), %edi
        movl    INPSTR(%esp), %esi
        movl    CPYLEN(%esp), %ecx
        cld                             # Forwards
        repe    cmpsb
        jz      1f                      # Good compare
        incl    %eax                    # Bad compare
1:      popl    %edi
        popl    %esi
        popl    %ecx
        ret
.size   memcmp,.-memcmp
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       If the first parameter is a pointer to a string, strtok
#       returns that string value up-to, but not including the
#       delimiter supplied as the second parameter. If the
#       first parameter is a NULL, then strtok returns the
#       next token in the previously-supplied string. If no
#       new tokens are found, strtok returns NULL.  This procedure
#       saves the pointer to the next token in a secret location
#       called __tok, external to this procedure.
#
#
STR = 0x0c
DEL = 0x10
.extern __tok   .long
.global strtok
.type   strtok,@function
strtok: pushl   %ebx
        pushl   %esi
        xorl    %ebx, %ebx              # Default NULL-pointer
        movl    STR(%esp), %esi         # Get string location
        orl     %esi, %esi              # Check for a NULL
        jnz     1f                      # Valid pointer
        movl    (__tok), %esi           # Get saved pointer
        orl     %esi, %esi              # Check for NULL
        jz      4f                      # All done
1:      movl    %esi, %ebx              # Save starting location
        movl    DEL(%esp), %eax         # Get delimter pointer
        movb    (%eax), %ah             # Get byte at that location
        cld                             # Forwards
2:      lodsb                           # Get byte
        orb     %al,%al                 # Check for end of string
        jz      3f                      # It's the end
        cmpb    %ah, %al                # See if the token
        jnz     2b                      # Not yet
        movl    %esi, (__tok)           # Save pointer to next
        movb    $0,-1(%esi)             # Terminate string
        jmp     4f
3:      cmpl    $0x00,(__tok)           # First time through?
        movl    $0x00,(__tok)           # Set NULL pointer
        jnz     4f                      # No
        xorl    %ebx,%ebx               # Get a zero
4:      movl    %ebx, %eax              # Return in EAX
        popl    %esi
        popl    %ebx
        ret
.size   strtok,.-strtok
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This clears the memory area pointed to by the first input
#       parameter.
#
#       This is a callable procedure like bzero(), which was replaced
#       in POSIX with memset(). Since it is not in-line, the code is
#       shorter. I also runs at the same speed.
#
#       void clr(void *mem, size_t len);
#       void bzero(void *mem, size_t len);
#

CLRBUF  = 0x0c
CLRLEN  = 0x10
.global clr
.type clr,@function
.global bzero
.type bzero,@function
.align  0x04
bzero:
clr:    pushl   %ecx
        pushl   %edi
        movl    CLRBUF(%esp),%edi
        movl    CLRLEN(%esp),%ecx
        xorl    %eax,%eax
        cld
        shrl    $1,%ecx                 # Create words
        pushf                           # Save possible byte
        shrl    $1,%ecx                 # Create longwords
        rep     stosl                   # Store longwords
        adc     %ecx,%ecx               # Possible word
        rep     stosw                   # Store odd word
        popf                            # Flags from first shift
        adc     %ecx,%ecx               # Possible byte
        rep     stosb                   # Do last byte
        popl    %edi
        popl    %ecx
        ret
.size   bzero,.-bzero
.size   clr,.-clr
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This writes a byte value to the memory area pointed to by the
#       first input parameter.
#
#       void memset(void *mem, int value, size_t len);
#

SETBUF  = 0x0c
SETVAL  = 0x10
SETLEN  = 0x14

.global memset
.type memset,@function
.align  0x04
memset: pushl   %ecx
        pushl   %edi
        movl    SETVAL(%esp),%eax       # AL = Val
        movb    %al, %ah                # AH = AL = Val
        movl    %eax, %ecx
        shll    $0x10, %ecx
        orl     %ecx, %eax              # All bytes contain Val
        movl    SETBUF(%esp),%edi
        movl    SETLEN(%esp),%ecx
        pushl   %edi                    # The buffer area
        cld
        shrl    $1,%ecx                 # Create words
        pushf                           # Save possible byte
        shrl    $1,%ecx                 # Create longwords
        rep     stosl                   # Store longwords
        adc     %ecx,%ecx               # Possible word
        rep     stosw                   # Store odd word
        popf                            # Flags from first shift
        adc     %ecx,%ecx               # Possible byte
        rep     stosb                   # Do last byte
        popl    %eax                    # Return value was the buffer
        popl    %edi
        popl    %ecx
        ret
.size   memset,.-memset
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  This converts an input BCD number to binary. BCD contains 0->9 in
#  both the upper and lower nibbles of an input byte for a total
#  dynamic range of 0 to 99.
#
VALUE=0x04

.global bcd2bin
.type bcd2bin,@function
.align  0x04
bcd2bin:
        movl    VALUE(%esp), %eax       # Get input value.
        movb    %al,%ah                 # Get a copy
        andb    $0x0f, %al              # Whack that off
        shrb    $0x03, %ah              # Put (* 2) into low nibble
        andb    $0x1e, %ah              # Clean up extra bits
        addb    %ah,%al                 # Sum partial product
        shlb    $2,%ah                  # Times 8
        addb    %ah,%al                 # Sum result + previous * 2
        andl    $0xff,%eax              # Clean up
        ret
.size   bcd2bin,.-bcd2bin
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  This converts an input binary number to BCD.
#
.global bin2bcd
.type bin2bcd,@function
.align  0x04
bin2bcd:
        movl    VALUE(%esp), %eax       # Get input value.
        aam
        shlb    $4,%ah
        orb     %ah,%al
        andl    $0xff,%eax              # Clean up
        ret
.size   bin2bcd,.-bin2bcd
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  Given an input buffer pointer and an integer value, this procedure
#  converts it to an ASCII string. It returns the address of the buffer.
#
#       char *itos(char *buffer, int value);
#
DST = 0x14
VAL = 0x18

.global itos
.type   itos,@function
.align  0x04
itos:   pushl   %ebx
        pushl   %ecx
        pushl   %edx
        pushl   %edi
        cld
        movl    DST(%esp), %edi # Set destination pointer
        movl    VAL(%esp), %eax # Get input value
        orl     %eax, %eax      # Check sign
        jns     1f              # It is positive
        negl    %eax            # Negative, make positive
        movb    $0x2d,(%edi)    # Insert dash into output string
        incl    %edi            # Bump pointer
1:      movl    $10,%ebx        # Decimal division
        xorl    %ecx, %ecx      # Loop counter
2:      incl    %ecx            # Loop counter
        xorl    %edx,%edx       # Clear register and flags
        divl    %ebx            # Divide by 10
        pushl   %edx            # Save 0 to 9 on the stack
        orl     %eax,%eax       # Did we get to zero?
        jnz     2b              # Nope, continue
3:      popl    %eax            # Get last to first from stack
        addb    $0x30, %al      # ASCII decimal bias
        stosb                   # Put into output string
        loop    3b              # Continue for all
        movb    $0x00,(%edi)    # Terminating null-byte
        movl    DST(%esp), %eax # Return buffer address
        popl    %edi
        popl    %edx
        popl    %ecx
        popl    %ebx
        ret
.size   itos,.-itos
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#   This writes a string to standard output.
#
.global write_str
.type   write_str,@function
.align  0x04
write_str:
        pushl   0x04(%esp)              # Pointer to string on stack
        call    strlen                  # Get length in eax
        pushl   %eax                    # Length
        pushl   0x0c(%esp)              # String
        pushl   $1                      # Standard output FD
        call    write
        addl    $0x10, %esp             # Level stack
        ret
.size   write_str,.-write_str
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This receives a pointer to a long-long plus a divisor.
#       It returns the remainder in eax, plus it returns the
#       quotent via the pointer also.
#
.global __lldiv
.type   __lldiv,@function
.align  0x04
__lldiv:
        pushl   %ebx
        movl    0x08(%esp), %ebx        # Pointer to long-long
        movl    0x0c(%esp), %ecx        # Divisor
        xorl    %edx,%edx               # Clear remainder and CY
        movl    0x04(%ebx), %eax        # Get the high word
        divl    %ecx, %eax              # Divide high word
        movl    %eax, 0x04(%ebx)        # High part of quotent
        movl    (%ebx), %eax            # Low word
        divl    %ecx, %eax              # Divide low word
        movl    %eax, (%ebx)            # Low part of quotent
        movl    %edx, %eax              # Return remainder
        popl    %ebx
        ret
.size   __lldiv,.-__lldiv
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This gets a buffer of ASCII text and returns a long-long
#       signed decimal value.
#
PARM = 0x0c
.global atoll
.type   atoll,@function
.global __strtoll_internal
.type   __strtoll_internal,@function
.align  0x04
__strtoll_internal:
atoll:  pushl   %ebx
        pushl   %esi
        movl    PARM(%esp), %esi        # Get buffer pointer
        xorl    %eax, %eax              # Zero
        movl    %eax, %edx              # Zero
        movl    %eax, %ecx              # Zero
        movl    %eax, %ebx              # Zero
        movl    %eax, PARM(%esp)        # Use as a flag
        cld                             # Forward
#
#       Filter the white-space and check for a sign. The standard
#       says that '+' is ignored as well as any white-space.
#
1:      movb    (%esi), %al             # Get buffer byte
        orb     %al,%al                 # Check for end
        jz      6f                      # Nothing left
        cmpb    $0x2b, %al              # A plus sign
        jz      4f                      # Yes, continue
2:      cmpb    $0x2d, %al              # A minus sign?
        jnz     3f                      # No
        decl    PARM(%esp)              # Show minus
        jmp     4f                      # Continue
3:      cmpb    $0x20, %al              # Check for white space
        jnbe    5f                      # Maybe a number
4:      inc     %esi                    # Next buffer location
        jmp     1b                      # Continue
#
#       Okay, leading white-space is gone and a possible '-' is
#       resolved. The first non-numeric byte ends this loop.
#
5:      lodsb                           # Get byte
        subb    $0x30, %al              # Remove ASCII bias
        jb      6f                      # Less than '0'
        cmpb    $0x0a, %al              # Check upper limits
        jnb     6f                      # Not a number
#
#       We have a number. Multiply previous number by 10 and
#       add the new number to it.
#
        pushl   %edx                    # Save times 1 high word
        pushl   %ecx                    # Save times 1 low word
        shll    $1, %ecx                # Times 2
        rcll    $1, %edx                # Propigate
        shll    $1, %ecx                # Times 4
        rcll    $1, %edx                # Propigate
        popl    %ebx                    # Get times 1 low word
        addl    %ebx, %ecx              # Times 5
        popl    %ebx                    # Get times 1 high word
        adcl    %ebx, %edx              # Propigate
        shll    $1, %ecx                # Times 10
        rcll    $1, %edx                # Propigate
        addl    %eax, %ecx              # Add new value
        adcl    $0, %edx                # Propigate
        jmp     5b                      # Continue
6:      movl    %ecx, %eax              # Low word, high in %edx
        cmpl    $0,PARM(%esp)           # Get minus flag
        jz      7f                      # It's positive
        notl    %edx                    # Invert the high bits
        notl    %eax                    # Invert the low bits
        addl    $1, %eax                # Add 1
        adcl    $0, %edx                # And propigate
7:      popl    %esi
        popl    %ebx
        ret
.size   atoll,.-atoll
.size   __strtoll_internal,.-__strtoll_internal
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This gets a pointer to a string that needs to be found in
#       the environment. The environment is searched and, if it's
#       found, a pointer is returned to the byte after the '='
#       character. In the event that it's not found, a NULL is
#       returned.
#
ENVP = 0x10
.align  0x04
getenv: pushl   %ebx
        pushl   %esi
        pushl   %edi
        cld                             # Forwards
        movl    ENVP(%esp), %eax        # Pointer to string
        pushl   %eax                    # Parameter to strlen
        call    strlen                  # Get the length
        movl    %eax, %ecx              # Length is in ecx
        popl    %edx                    # Beginning of string
        movl    (__environ), %ebx       # Environment pointer
1:      movl    (%ebx), %esi            # Get environment string
        orl     %esi, %esi              # See if a NULL
        jz      3f                      # It's the end
        leal    0x04(%ebx), %ebx        # Ready next environment pointer
        pushl   %ecx                    # Save scan length
        movl    %edx, %edi              # String to scan
        pushl   %esi                    # New string
        call    strlen                  # See if we scan it
        addl    $0x04, %esp             # Level stack
        cmpl    %ecx, %eax              # Check length
        jc      2f                      # No, not long enough
        repz    cmpsb                   # Compare strings
2:      popl    %ecx                    # Restore byte count
        jnz     1b                      # No compare yet
        cmpb    $0x3d,(%esi)            # Really the string?
        jnz     1b                      # Nope, continue
        incl    %esi                    # Get past the '='
3:      movl    %esi, %eax              # Return pointer value
        popl    %edi
        popl    %esi
        popl    %ebx
        ret
.size   getenv,.-getenv
.type   getenv,@function
.global getenv
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This sets the errno wherever it is (it's sitting on the stack).
#
.extern __errnop  .long
.align  0x04
__set_errno:
        movl    (__errnop), %eax        # Get location
        movl    0x04(%esp), %ecx        # Get errno value
        movl    %ecx, (%eax)            # Put it into area
        ret
.size   __set_errno,.-__set_errno
.type   __set_errno,@function
.global __set_errno
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This returns the location of global errno.
#
__errno_location:
        movl    (__errnop), %eax
        ret
.size   __errno_location,.-__errno_location
.type   __errno_location,@function
.global __errno_location
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       I don't know why this happens this way, but readdir()
#       needs this dumb procedure.
#
.align  0x04
unlikely:
        movl    0x04(%esp), %eax        # Get passed parameter
        ret                             # And return with it!
.size   unlikely,.-unlikely
.type   unlikely,@function
.global unlikely
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       This gets the errno from wherever we hid it.
#
.align  0x04
__get_errno:
        call    __errno_location        # Get address of variable
        movl    (%eax), %eax            # Return with it.
        ret
.size   __get_errno,.-__get_errno
.type   __get_errno,@function
.global __get_errno
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
__get_sp:
        movl    %esp, %eax
        subl    $0x04, %eax             # Return address
        ret
.size   __get_sp,.-__get_sp
.type   __get_sp,@function
.global __get_sp
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#       Various fixups.
#
.align  0x04
strtol: jmp     simple_strtol
.align  0x04
__strtol_internal:
        jmp     simple_strtol
.align  0x04
__strtoul_internal:
        jmp simple_strtoul
.align  0x04
putc:   jmp fputc
.align  0x04
_IO_putc:       jmp fputc
.align  0x04
__strdup:       jmp strdup
.align  0x04
strcoll:        jmp strcmp
.align  0x04
__strtod_internal:      jmp     strtod
.global strtol
.type   strtol,@function
.global __strtol_internal
.type   __strtol_internal,@function
.global __strtoul_internal
.type   __strtoul_internal,@function
.global putc
.type   putc,@function
.global _IO_putc
.type   _IO_putc,@function
.global __strdup
.type   __strdup,@function
.global strcoll
.type   strcoll,@function
.global __strtod_internal
.type   __strtod_internal,@function
.end
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

