#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#
#  Copyright(c)  2005  Richard B. Johnson
#
#  This program may be distributed under the GNU Public License
#  version 2, as published by the Free Software Foundation, Inc.,
#  59 Temple Place, Suite 330 Boston, MA, 02111.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  File makeaddr.S             Created FEB-03-2000     Richard B. Johnson
#
#  This file contains assembly language to take the "dotted quad"
#  representation of TCP/IP addresses and convert to a longword.
#  It is in assembly  to save space.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#
#  Given an input string containing an 8-bit ASCII decimal string, and
#  a pointer to write the 8 bit decimal value, this procedure returns
#  a pointer that is one character past the terminating character after
#  the conversion is complete.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

INBUF  = 0x0c
WRPTR  = 0x10
.section .text
.type do8bits,@function
.align  0x04
do8bits:
        pushl %ebx 
        pushl %esi
        movl INBUF(%esp), %esi
        movl WRPTR(%esp), %ebx
        xorl %edx, %edx  # Zero accumulator
1:      lodsb            # Get buffer byte
        cmpb $0x20, %al  # Eat up any spaces
        jz   1b          # Don't type nicely 
        subb $0x30, %al  # Subtract ASCII bias
        jb   2f          # Not a number, done
        cmpb $0x0a, %al  # Check maximum value
        jnb  2f          # Not a number, done
        shlb %dl         # Times 2 
        movb %dl, %dh    # Save times 2
        shlb $0x02, %dl  # Times 8
        addb %dh, %dl    # Times 10
        addb %al, %dl    # Accumulate
        jmp 1b           # Continue until done 
2:      movb %dl, (%ebx) # Put byte in buffer
        movl %esi, %eax
        popl %esi
        popl %ebx
        ret

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#
#  This takes an input string in the "dotted-quad" format and returns
#  the result as an unsigned longword in network order.
#

INSTR = 0x08 
.global makeaddr
.type makeaddr,@function
.align  0x04
makeaddr:
        pushl %ebx
        movl INSTR(%esp), %eax
        subl $0x04, %esp   # Room for a variable
        movl %esp, %ebx    # Point to the variable
        movl $0, (%ebx)    # Start with zero 
        movl $0x04, %ecx   # 4 bytes to use 
3:      pushl %ecx         # Save count
        pushl %ebx         # Address to write byte 
        pushl %eax         # String address
        call do8bits       # Call procedure above
        addl $0x08, %esp   # Level stack
        rorl $0x08, (%ebx) # Rotate result to right
        popl %ecx          # Restore count
        loop 3b            # Continue for 4 bytes
        movl (%ebx), %eax  # Value to return
        addl $0x04, %esp   # Level stack
        popl %ebx
        ret
.size   makeaddr,.-makeaddr
.end
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
