//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//
//  Copyright(c)  2005  Richard B. Johnson
//
//  This program may be distributed under the GNU Public License
//  version 2, as published by the Free Software Foundation, Inc.,
//  59 Temple Place, Suite 330 Boston, MA, 02111.
//
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//
//   File checker.c     Created 11-NOV-2002     Richard B. Johnson
//   Checks the tiny 'C' runtime library.
//

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <utime.h>
#include <termios.h>
#include <sys/utsname.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <sys/mount.h>
#include <sys/file.h>
#include <sys/times.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/resource.h>
#include <errno.h>
#include <libgen.h>
#include <netdb.h>
#include <time.h>
#include <ctype.h>

#define TEST_LEN 0x2000000
#define SORT_LEN (24 * 1000)

struct sysinfo {
    long uptime;
    unsigned long loads[3];
    unsigned long totalram;
    unsigned long freeram;
    unsigned long sharedram;
    unsigned long bufferram;
    unsigned long totalswap;
    unsigned long freeswap;
    unsigned short procs;
    char _f[22];
    };

typedef int(*cmp_t)(const void *, const void *);
extern void __set_errno(int);
extern void write_str(const char*);
extern int Mknod(const char *, mode_t, dev_t );
extern int sysinfo(struct sysinfo *sip);
extern char *itos(char *buf, int val);
extern int _break(int, int);      /* Returns ENOSYS */
extern int stty(int, int);
extern int gtty(int, int);
extern int mpx(int, int);
extern int prof(int);
extern int lock(const char *, int);
extern int ustat(int, struct stat *sb);
extern pid_t getpgid(pid_t);
extern int uselib(const char *lib);
extern int swapon(const char *, int);
extern int swapoff(const char *);
extern int sys_readdir(int fd, void *sd, size_t len);
extern size_t strnlen(const char *cp, size_t len);
extern int syslog(int, void *, int);
void dummy(int ignored) { }
const char fname0[]="test.dat";
const char fname1[]="test.DAT";
const char textfile[]="test.txt";
extern const char *strnchr(const char *, int, size_t);
extern char *strtoupr(char *);
extern void *__get_sp(void);

extern int __lldiv(long long *val, int num);
extern long __bss_start;
extern long _end;
extern long __data_start;

char foo[19];  /* It's in the .bss section */


void Aexit()
{
    puts("First atexit() function called");
}

void Bexit()
{
    puts("Second atexit() function called");
}

int main(int argc, char *argv[], char *envp[])
{
    FILE *file;
    char *env[0x04];
    char buf[0x400];
    struct hostent *hp;
    struct servent *sp;
    char *cp, *rp;
    int len, fd, ret;
    time_t tim;
    size_t i, j, k, l;
    caddr_t addr;
    struct timeval tv;
    struct timezone tz;
    struct timespec *ts;
    struct utimbuf ut;
    struct sigaction sa;
    long long checkmate;
    fd_set rfds, wfds, efds;
    double x;
    printf("Stack is at %p\n", __get_sp());
    printf("Page size = %08lx\n", getpagesize());
    printf("isatty = %d\n", isatty(0));
    printf("isascii %d\n", isascii('a'));
    printf("isalnum %d\n", isalnum('a'));
    printf("isalnum %d\n", isalnum(0));
    printf("isalpha %d\n", isalpha('a'));
    printf("iscntrl %d\n", iscntrl('a'));
    printf("isdigit %d\n", isdigit('a'));
    printf("islower %d\n", islower('a'));
    printf("isgraph %d\n", isgraph('a'));
    printf("isprint %d\n", isprint('a'));
    printf("ispunct %d\n", ispunct('a'));
    printf("isspace %d\n", isspace('a'));
    printf("isupper %d\n", isupper('a'));
    printf("isxdigit %d\n", isxdigit('a'));
    printf("isspace %d\n", isspace(' '));
    printf("argc = %d\n", argc);
    printf("argv[0] = %s\n", argv[0]);
    printf("argv[1] = %s\n", argv[1]);
    printf("UID = %d\n", getuid());
    printf("%s\n", strchr("\"`", 0));
    fputs("Hello World!\n", stderr);
    fputs("Hello World!\n", stdout);

    gethostname(buf, sizeof(buf));
    printf("Host name = %s\n", buf);
    hp = gethostbyname(buf);
    printf("Host address = %s\n", hp->h_addr);
    hp = gethostbyname(hp->h_addr);
    printf("Host address = %s\n", hp->h_addr);
    hp = gethostbyaddr(hp->h_addr, strlen(hp->h_addr), AF_INET);
    printf("Host name = %s\n", hp->h_name);
    sp = getservbyname("ftp", "tcp");
    printf("service FTP is %d, protocol %s\n", htons(sp->s_port), sp->s_proto);
    sp = getservbyname("ftp", NULL);
    printf("service FTP is %d, protocol %s\n", htons(sp->s_port), sp->s_proto);
    printf("Password was %s\n", getpass("Prompt: "));

//    hp = gethostbyname("Linux");



    printf("Temporary name = %s\n", tmpnam(NULL));
    printf("Basename = %s\n", basename("/1/2/3/4/5/6/7/8/name"));
    printf("Basename = %s\n", basename("/name"));
    printf("Basename = %s\n", basename("name"));
    printf("Basename = %s\n", basename("///name"));
    printf("Basename = %s\n", basename("/"));


    for(i = ' '; i <= 'z'; i++)
        buf[i] = (char) i;
    buf[i] = 0x00;
    puts(strtoupr(&buf[' ']));

    for(i=0; i< 0x10; i++)
    {
        if((cp = malloc(TEST_LEN)) == NULL)
        {
            fprintf(stderr,"Not enough memory\n");
            exit(EXIT_FAILURE);
        }
        bzero(cp, TEST_LEN);
        if((rp = malloc(TEST_LEN)) == NULL)
        {
            fprintf(stderr,"Not enough memory\n");
            exit(EXIT_FAILURE);
        }
        bzero(rp, TEST_LEN);
        free(rp);
        bzero(cp, TEST_LEN);
        free(cp);
    }
    printf("Break = %p\n", sbrk(0));
    printf("Should read something %p\n", strnchr("123456", '5', 6));
    printf("Should read something %p\n", strnchr("123456", '5', 5));
    printf("Should read 0 %p\n", strnchr("123456", '5', 4));
    printf("errno = %p\n", &errno);
    printf("errno = %p\n", __errno_location());
    __set_errno(EINVAL);
    perror("Just checking!");
    errno = EINVAL;
    perror("Just checking!");
    errno = 0;
    perror("Just checking!");

    x = atof("123456789.123456789");
    printf("%f\n", x);
    printf("%s\n", getenv("TERM"));
    printf("%s\n", getenv("VISUAL"));
    printf("%s\n", getenv("TZ"));
    setenv("VISUAL", "whatever", 1);
    printf("%s\n", getenv("VISUAL"));
    setenv("VISUAL", "vi", 1);
    printf("%s\n", getenv("VISUAL"));
    setenv("VISUAL", "/bin/vi", 1);
    printf("%s\n", getenv("VISUAL"));
    setenv("Z-ANTIDISESTABLISHMENTARIANISM", "HoppalongCassidy", 1);
    printf("%s\n", getenv("VISUAL"));
    printf("%s\n", getenv("Z-ANTIDISESTABLISHMENTARIANISM"));

    file = popen("set>Environ", "r");
    while(fgets(buf, 0x400, file) != NULL)
        printf(buf);
    pclose(file);
    fprintf(stdout,"Sorting a large array....");
    fflush(stdout);
    if((cp = malloc(SORT_LEN)) == NULL)
    {
        fprintf(stderr, "Malloc failed\n");
        exit(EXIT_FAILURE);
    }
    rp = cp;
    k = 0;
    for(i = 0; i< SORT_LEN; i += j)
    {
        j = sprintf(rp,"%08x%08x%08x SORT THE ARRAY\n", rand(), rand(), rand());
        rp += j;
        k++;
    }
    *rp = 0x00;
    qsort(cp, k, j, (cmp_t)strcmp);
    fprintf(stdout,"done!\n");
    write(STDOUT_FILENO, cp, SORT_LEN);
    free(cp);
//    abort();
    system("/bin/ls -laR");
    puts(strdup("Hello from strdup()"));
    puts(strpbrk("Hello from strpbrk()", "123456789o87654321"));
    puts(strpbrk("Hello from strpbrk()", "9876543210H0123456789"));
    fflush(stdout);

    sscanf("1 2 3 4", "%d%d%d%d", &i, &j, &k, &l);
    printf("Should print 1 2 3 4 -> %d %d %d %d\n", i, j, k, l);
    puts("");
    strcpy(buf, "One Two Three");
    printf("Should print One : ");
    printf("%s\n", strtok(buf, " "));
    printf("Should print Two : ");
    printf("%s\n", strtok(NULL, " "));
    printf("Should print Three : ");
    printf("%s\n", strtok(NULL, " "));
    printf("Should print (NULL) : ");
    printf("%s\n", strtok(NULL, " "));
    strcpy(buf, "One,Two,Three,Four,Five");
    printf("Should print One : ");
    printf("%s\n", strtok(buf, ","));
    printf("Should printf Two : ");
    printf("%s\n", strtok(NULL, ","));
    printf("Should print Three : ");
    printf("%s\n", strtok(NULL, ","));
    printf("Should print Four : ");
    printf("%s\n", strtok(NULL, ","));
    printf("Should print Five : ");
    printf("%s\n", strtok(NULL, ","));
    printf("Should print (NULL) : ");
    printf("%s\n", strtok(NULL, ","));
    strcpy(buf, "One,Two,Three,Four,Five");
    printf("Should print (NULL) : ");
    printf("%s\n", strtok(buf, " "));
    puts("");

    file = fopen(textfile, "w+");
    printf("%p\n", file);
    ret = fileno(file);
    printf("The file number was %d\n", ret);
    fprintf(stdout, "Lock the file = %d\n", flock(ret, LOCK_EX));

    fprintf(file, "This is test0\nThis is test1\nThis is test2\n");
    fprintf(file, "This is test3\nThis is test4\nThis is test5\n");
//    fputs("This is test0\nThis is test1\nThis is test2\n", file);
//    fputs("This is test3\nThis is test4\nThis is test5\n", file);
    rewind(file);
    fprintf(stdout, "Unlock the file = %d\n", flock(ret, LOCK_UN));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fclose(file);
/*
 *  The following should check error conditions.
 */
    fprintf(stdout, "%s\n", fgets(buf, sizeof(buf), file));
    ret = ferror(file);
    fprintf(stdout, "Error was %d (%s)\n", ret, strerror(errno));

    file = fopen(textfile, "r");
    printf("%p\n", file);
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fprintf(stdout, "%s", fgets(buf, sizeof(buf), file));
    fclose(file);
    fprintf(stdout, "Hello from stdout\n");
    fprintf(stderr, "Hello from stderr\n");

#if 0
    fscanf(stdin, "%d %d %d %d", &i, &j, &k, &l);
    printf("%d %d %d %d\n", i, j, k, l);
    fscanf(stdin, "%d %d %d %d", &i, &j, &k, &l);
    printf("%d %d %d %d\n", i, j, k, l);
#endif
    printf("%s\n", realpath("../../etc", buf));
    fputs("Hello again from stdout!\n", stdout);
    fputs("Hello again from stderr!\n", stderr);
    fputc('.', stdout);
    fputc('.', stdout);
    fputc('.', stdout);
    putc('.', stdout);
    putc('.', stdout);
    putc('.', stdout);
    putc('\n', stderr);
    putchar('\n');
    if((cp = malloc(TEST_LEN)) == NULL)
    {
        printf("Malloc doesn't work, %s\n", strerror(errno));
        exit(EXIT_FAILURE);
    }
    memset(cp, 0xff, TEST_LEN);
    free(cp);

    printf("%p\n", memmove(buf, "123456789\0", 10));
    printf("%p\n", buf);
    puts(buf);
/*
 *  memmove() does overlaping copies.
 */
    memmove(buf, buf, 10);
    memmove(&buf[1], buf, 10);
    memmove(buf, &buf[1], 10);
    puts(buf);

/*
 *  Check all the corner cases.
 */

    printf("%s\n", strrchr("123456789123456789", '1'));
    printf("%s\n", strrchr("/long/path/name/filename", '/'));
    printf("%s\n", strrchr("/long/path/name/filename", 'z'));
    printf("%s\n", strchr("123456789123456789", '1'));
    printf("%s\n", strchr("/long/path/name/filename", '/'));

/*
 *  Check all the corner cases.
 */
    printf("%s\n", strstr("123456789qwertyuiopHello!", "H"));
    printf("%s\n", strstr("123456789", "123456789"));
    printf("%s\n", strstr(" 123456789", "123456789"));
    printf("%s\n", strstr("  123456789", "123456789"));
    printf("%s\n", strstr("123456789", ""));
    printf("%s\n", strstr("123456789", "1"));
    printf("%s\n", strstr("123456789", "2"));
    printf("%s\n", strstr("123456789", "3"));
    printf("%s\n", strstr("123456789", "4"));
    printf("%s\n", strstr("123456789", "5"));
    printf("%s\n", strstr("123456789", "6"));
    printf("%s\n", strstr("123456789", "7"));
    printf("%s\n", strstr("123456789", "8"));
    printf("%s\n", strstr("123456789", "9"));

    printf("bss_start = %p\n", &__bss_start);
    printf("bss end   = %p\n", &_end);
    len = (&_end - &__bss_start);
    printf(".bss length = %d\n", len);

    printf("argc = %d  argv[0] = %s\n", argc, argv[0]);
    printf("envp[0] = %s\n", envp[0]);
    printf("envp[1] = %s\n", envp[1]);

    printf("%u\n", strlen("1234"));
    printf("%u\n", strnlen("1234", 10));
    printf("%u\n", strnlen("1234", 1));
    printf("%u\n", strnlen("1234", 2));
    printf("%u\n", strnlen("1234", 3));
    printf("%u\n", strnlen("1234", 4));
    printf("%u\n", strnlen("1234", 5));
    printf("%u\n", strnlen("1234", 6));
    printf("%u\n", strlen("1"));
    printf("%u\n", strlen(""));


    strcpy(buf, "1234");
    strcat(buf, "5678");
    puts(buf);
    strncpy(buf, "1234", 10);
    strncat(buf, "5678", 10);
    puts(buf);


    printf("1234567890\n");
    printf("%d\n", 1234567890);
    printf("%d\n", -1234567890);
    puts(*__environ);
    puts(envp[0]);
    errno = 0x00;
    printf("call putenv() return %d\n", putenv("TERM=YES"));
    perror("putenv");
    puts(getenv("TERM"));

    do { atexit(Aexit);
       } while (!atexit(Bexit));
    for(i=0; i< sizeof(foo); i++)
        if(foo[i] != 0)
            printf("BSS not initialized correctly at %u\n", i);
#undef strcmp
    printf("%d\n", strcmp("1234", "1234"));
    printf("%d\n", strcmp("1234     ", "1234"));
    printf("%d\n", strcmp("1234", "0234"));
    printf("%d\n", strcmp("0234", "1234"));

    printf("%d\n", strncmp("1234", "1234", 10));
    printf("%d\n", strncmp("1234     ", "1234", 4));
    printf("%d\n", strncmp("1234", "0234", 2));
    printf("%d\n", strncmp("0234", "1234", 4));

    ts = (struct timespec *)buf;
    ts->tv_sec = 1;
    ts->tv_nsec = 0;
    nanosleep(ts, ts);
    switch(fork())
    {
    case 0:
        printf("Child with pid = %d\n", getpid());
        (void)sleep(1);
        return 66;
    case -1:
        perror("fork");
        break;
    default:
        wait((int *)&i);
//        waitpid(-1, (int *)&i, 0);
        i >>= 0x08;
        if(i != 66)
            fprintf(stderr, "Child returned wrong value %d\n", i);
        else
            fprintf(stdout, "Child returned correct value %d\n", i);
    }

    if(tcgetattr(0, (struct termios *)buf) < 0)
        perror("tcgetattr");
    if(tcsetattr(0, TCSANOW, (struct termios *)buf) < 0)
        perror("tcsetattr");

    tcsetpgrp(0, tcgetpgrp(0));
    if(sysinfo((struct sysinfo *)buf) < 0)
        perror("sysinfo");

    checkmate = atoll("     + 1234567890123456789");
    printf("%lld\n", checkmate);
    checkmate = atoll("  -    1234567890123456789");
    printf("%lld\n", checkmate);
    checkmate = atoll("  -1234567890123456789");
    printf("%lld\n", checkmate);
    checkmate = atoll("  -1234567890123456789alfdhlas");
    printf("%lld\n", checkmate);
    checkmate = atoll("  -1234567890123456789AB");
    printf("%lld\n", checkmate);
    checkmate = atoll("  -\t\t\t\t1234567890123456789AB");
    printf("%lld\n", checkmate);
    printf("%d\n", atoi("12345"));

    checkmate = 0xffffffffffffffffLL;
    while(checkmate)
        printf("%llx", __lldiv(&checkmate, 0x10));
    printf(" hex = ");
    checkmate = 0xffffffffffffffffLL;
    while(checkmate)
        printf("%d", __lldiv(&checkmate, 10));
    printf(" dec = ");
    checkmate = 0xffffffffffffffffLL;
    while(checkmate)
        printf("%d", __lldiv(&checkmate, 8));
    puts(" octal");
    puts(" backwards!");

    for(i=0; i< 130; i++)
        printf("%d = %s\n",i,  strerror(i));
    fd = open("/dev/mem", O_RDWR);
    addr = (caddr_t)0x2000;
    addr = mmap(addr, 0x2000, PROT_READ, MAP_SHARED, fd, 0x2000);
    munmap(addr, 0x2000);

#if 0
    len = read(0, buf, sizeof(buf));
    write(1, buf, len);
#endif
    printf("Open/create a file, %s\n", fname0);
    fd = open(fname0, O_RDWR|O_CREAT|O_TRUNC, 0644);
    printf("Return value was %d\n", fd);
    printf("Write to file %d\n", write(fd, "0123456789\n", 11));
    printf("Lseek to one byte %ld\n",lseek(fd, 1, SEEK_SET));
    printf("Read file, %d\n",(len = read(fd, buf, sizeof(buf))));
    buf[len] = 0x00;
    printf(buf);
    printf("Lseek to end %ld\n",lseek(fd, 0, SEEK_END));
    printf("Close file %d\n", close(fd));

    waitpid(-1, &len, WNOHANG);
    printf("Create file %d\n", (fd = creat(fname0, 0644)));
    printf("Close file %d\n", close(fd));
    printf("link file %s to %s %d\n", fname0, fname1, link(fname0, fname1));
    printf("unlink file %s %d\n", fname0, unlink(fname0));
    env[0] = "/bin/bash";
    env[1] = NULL;
//    execve(env[0], env, __environ);
    printf("getcwd, %p %s\n", getcwd(buf, sizeof(buf)), buf);
    printf("chdir %s %d\n", buf, chdir(buf));
    if(chdir("/etc/1/2/3/4/5/6/7/8"))
        perror("chdir");
    printf("chdir to bad directory %d\n", chdir("/etc/1/2/3/4/5/6/7/8"));
    tim = 0;
    printf("Get time %08lx\n",(tim = time(&tim)));
    stime(&tim);
    ut.actime  = tim;
    ut.modtime = tim;
    ret = Mknod("MYNODE", S_IFCHR|0644, (1 << 0x08) | 0x1);
    printf("Return value of mknod was %d\n", ret);
    if((ret = mknod("MYNODE", S_IFCHR|0644, (1 << 0x08) | 0x1)) < 0)
        perror("mknod");
    printf("Return value of mknod when one exists was %d\n", ret);
    printf("chmod %d\n", chmod(fname1, 0777));
    printf("chown %d\n", chown(fname1, 1,2));
    printf("lchown %d\n", lchown(fname1, 2,0));
    printf("lstat %d\n", lstat(fname1, (struct stat *) buf));
    printf("fstat %d\n", fstat(0, (struct stat *) buf));
    printf("utime %s %d\n", fname1, utime(fname1, &ut));
    printf("access %s %d\n", fname1, access(fname1, F_OK));
    printf("rename %s %d\n", fname1, rename(fname1, "zorx"));
    printf("unlink the file %d\n", unlink("zorx"));
    printf("access non-existant file %d\n", access(fname1, F_OK));
    printf("unlink the node %d\n", unlink("MYNODE"));
    _break(1,2);
    perror("_break");
    ret = getpid();
    strcpy(buf, "My pid is ");
    itos(&buf[strlen(buf)], ret);
    puts(buf);
    mount("/dev/sdc1", "/mnt", "ext2", MS_RDONLY, NULL);
    umount("/mnt");
    mount("/dev/sdc1", "/mnt", "ext2", MS_RDONLY, NULL);
    umount2("/mnt",0);
    setuid(getuid());
    stty(1,1);
    gtty(2,2);
    nice(0);
    ftime((struct timeb *) buf);
    sync();
    kill(0, 0);
    mkdir("ZAP", 0644);
    rmdir("ZAP");
    dup(0);
    pipe((int *)buf);
    times((struct tms *) buf);
    prof(0);
    brk((void*)(brk(0) + 0x800000));
    setgid(getgid());
    signal(SIGALRM, dummy);
    alarm(1);
    pause();
    getegid();
    geteuid();
    if(acct(fname0)< 0)
        perror("acct");
    if(lock(fname0, 1) < 0)
        perror("lock");
    ioctl(0, TCGETS, buf);
    fcntl(0, F_SETFL, fcntl(0, F_GETFL));
    if(mpx(1,2) < 0)
        perror("mpx");
    setpgid(getpid(), getpgid(getpid()));
    umask(umask(0));
    chroot("/");
    ustat(2, (struct stat *)buf);
    dup2(0, 9);
    getppid();
    getpgrp();
    memset(&sa, 0x00, sizeof(sa));
    sigaction(SIGALRM, NULL, &sa);
    sigaction(SIGALRM, &sa, NULL);
//    sigsetmask(siggetmask());
    uname((struct utsname *) buf);
    getrlimit(RLIMIT_DATA, (struct rlimit *)buf);
    setrlimit(RLIMIT_DATA, (struct rlimit *)buf);
    getrusage(RUSAGE_SELF, (struct rusage *)buf);
    gettimeofday(&tv, &tz);
    settimeofday(&tv, &tz);
    getgroups(sizeof(buf)/ sizeof(gid_t), (gid_t *) buf);
    getgroups(sizeof(buf)/ sizeof(gid_t), (gid_t *) buf);
    FD_ZERO(&rfds);
    FD_ZERO(&wfds);
    FD_ZERO(&efds);
    FD_SET(0, &rfds);
    FD_SET(1, &wfds);
    FD_SET(0, &efds);
    tv.tv_sec = 1;
    tv.tv_usec = 0;
    select(2, NULL, NULL, NULL, &tv);
    select(2, &rfds, &wfds, &efds, &tv);
    symlink("/", "top");
    readlink("top", buf, sizeof(buf));
    uselib("/lib/ld-linux.so.1");
    unlink("top");
    swapoff("/dev/sdb2");
    swapon("/dev/sdb2", 0);
    fd = open("/", O_RDONLY);
    sys_readdir(fd, (struct dirent *)buf, 0x400);
    (void)close(fd);
    if(wait4(getpid(), &fd, WNOHANG, NULL) < 0)
        perror("wait4");
    if(wait3(&fd, WNOHANG, NULL) < 0)
        perror("wait3");
    syslog(1, "Hello", 5);
    setsid();
    return 0;
}
