//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//
//    File getservbyname.c  Created 11-MAR-2009   Richard B. Johnson
//
//    Copyright(c) 2009         Richard B. Johnson
//

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>
#include <malloc.h>

static const char fname[]="/etc/services";
#define SERVNAME_LEN 0x20
#define NR_ALIAS 0x05
#define BUF_LEN 0x100

#define MAX(a, b) ((a) > (b) ? (a):(b))
#define MIN(a, b) ((a) < (b) ? (a):(b))
#define DEB(f)

struct servent *getservbyname(const char *name, const char *proto)
{
    FILE *file;
    struct servent *sp;
    char *tokens[NR_ALIAS + 1];
    char *buf;
    char *cp;
    size_t i, plen, nlen, tlen;
    int ret;
    if((file = fopen(fname, "r")) == NULL)
        return NULL;
    if((buf = malloc(BUF_LEN)) == NULL)
    {
        fclose(file);
        return NULL;
    }
    if(proto == NULL)
    {
        proto = buf;
        plen = 0;
    }
    else
        plen = strlen(proto);
    nlen = strlen(name);
    for(i=0; i < NR_ALIAS; i++)
        tokens[i] = malloc(SERVNAME_LEN); // Assume all good
    tokens[i] = NULL;                     // End of the list
    while(fgets(buf, BUF_LEN, file) != NULL)
    {
        if((cp = strchr(buf, '#')) == buf)
            continue;
        if(cp != NULL)
            *cp = (char) 0x00;
        if((cp = strchr(buf, '/')) != NULL)
            *cp = (char) ' ';
        for(i = 0; i< NR_ALIAS; i++)
            *tokens[i] = (char) 0x00;
        ret = sscanf(buf, "%s%s%s%s%s", tokens[0], tokens[1],
                           tokens[2], tokens[3], tokens[4] );
       DEB(printf("Tokens[0] = %s\n", tokens[0]));
       DEB(printf("Tokens[1] = %s\n", tokens[1]));
       DEB(printf("Tokens[2] = %s\n", tokens[2]));
       DEB(printf("Tokens[3] = %s\n", tokens[3]));
       DEB(printf("Tokens[4] = %s\n", tokens[4]));
       tlen = strlen(tokens[0]);
       if(memcmp(tokens[0], name, MAX(nlen, tlen)) == 0)
       {
           DEB(printf("Found as %s\n", tokens[0]));
           tlen = strlen(tokens[2]);
           if(memcmp(tokens[2], proto, MIN(plen, tlen)) == 0)
           {
              sp = malloc(sizeof(struct servent));
              sp->s_name    = tokens[0];
              sp->s_aliases = &tokens[3];
              sp->s_port    =  htons(atoi(tokens[1]));
              sp->s_proto   = tokens[2];
              fclose(file);
              free (buf);
              return sp;
           }
       } 
    }
    for(i=0; i < NR_ALIAS; i++)
        free(tokens[i]);
    free(buf);
    fclose(file);
    return NULL;
}
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
