//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//
//    File netdb.c  Created 18-MAR-2009    Richard B. Johnson
//
//    Copyright(c) 2009         Richard B. Johnson
//
//
//    gethostbyname, gethostbyaddr, sethostent, endhostent, herror,
//    hstrerror - get network host entry  
//

#include <stdio.h>
#include <netdb.h>
#include <malloc.h>
#include <string.h>
#include <linux/param.h>
#include <sys/socket.h>        /* for AF_INET */

#define BUF_LEN 0x1000
#define MAX_TOKENS 5

#define DEB(f)

#define ArraySize(a) (sizeof(a) / sizeof(a[0]))
static const char hfname[]="/etc/hosts";
extern char *strtoupr(char *);

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// The gethostbyname() function returns a structure of type hostent
// for the given host name. Here name is either a host name, or an
// IPv4 address in standard dot notation, or an IPv6 address in colon
// (and possibly dot) notation. (See RFC 1884 for the description of
// IPv6 addresses.)
// If name is an IPv4 or IPv6 address, no lookup is performed and
// gethostbyname() simply copies name into the h_name field and its
// struct in_addr equivalent into the h_addr_list[0] field of the
// returned hostent structure.
//
typedef struct {
    struct hostent hst;
    char   *addrlist[2];
    } MY_HP;

static void free_tokens(char **tokens, size_t start)
{
   size_t i;
   for(i = start; i< MAX_TOKENS; i++)
   {
       if(tokens[i] != NULL)
          free(tokens[i]);
   }
}
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//   Get the address of a host, returning it in a structure of
//   type "hostent."
//
struct hostent *gethostbyname(const char *name)
{
    char find[MAXHOSTNAMELEN];
    MY_HP *hp;
    FILE *file;
    char *buf;
    char *cp;
    size_t i;
    size_t len;
    char *tokens[MAX_TOKENS + 1];
    DEB(printf("gethostbyname\n"));
    if((file = fopen(hfname, "r")) == NULL)
        return NULL;
    if((buf = malloc(BUF_LEN)) == NULL)
    {
        fclose(file);
        return NULL;
    }
    len = strlen(name);
    strcpy(find, name);
    strtoupr(find);
    memset(tokens, 0x00, ArraySize(tokens));
    for(i = 0; i < MAX_TOKENS; i++)
    {
        if((tokens[i] = malloc(MAXHOSTNAMELEN)) == NULL)
        {
            free_tokens(tokens, 0);
            fclose(file);
            free(buf);
            return NULL;
        }
    }
    while(fgets(buf, BUF_LEN, file) != NULL)
    {
       if((cp = strchr(buf, '#')) == buf)
           continue;
       if(cp != NULL)
           *cp = (char) 0x00;
       strtoupr(buf);

       for(i=0; i< MAX_TOKENS; i++)
           *tokens[i] = (char) 0x00;
        sscanf(buf, "%s%s%s%s%s", tokens[0], tokens[1], tokens[2],
                                  tokens[3], tokens[4]);

        DEB(printf("token 0 = %s\n", tokens[0]));
        DEB(printf("token 1 = %s\n", tokens[1]));
        DEB(printf("token 2 = %s\n", tokens[2]));
        DEB(printf("token 3 = %s\n", tokens[3]));
        DEB(printf("token 4 = %s\n", tokens[4]));
        for(i= 0; i < MAX_TOKENS; i++)
        {
            if(memcmp(tokens[i], find, len) == 0)
            {
                hp = malloc(sizeof(MY_HP));
                DEB(printf("Found %s\n", tokens[i]));
                hp->addrlist[0]     = tokens[0];         // Only one address
                hp->addrlist[1]     = NULL;              // Null terminated
                hp->hst.h_name      = tokens[i];         // Name of host
                hp->hst.h_aliases   = tokens;	         // Alias list
                hp->hst.h_addrtype  = AF_INET;           // Only one allowed
                hp->hst.h_addr_list = hp->addrlist;      // list of addresses 
                hp->hst.h_length    = strlen(tokens[0]); // length of address
                len = i + 1;
                free_tokens(tokens, len);
                tokens[len] = NULL;
                fclose(file);
                free(buf);
                return &hp->hst; 
            }
         }
    }
    free_tokens(tokens, 0);
    fclose(file);
    free(buf);
    return NULL;
}
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// The gethostbyaddr() function returns a structure of type hostent
// for the given host address addr of length len and address type type.
// The only valid address type is currently AF_INET.
//
struct hostent *gethostbyaddr(const void *addr, socklen_t len, int type)
{
    char find[MAXHOSTNAMELEN];
    MY_HP *hp;
    FILE *file;
    char *buf;
    char *cp;
    size_t i;
    int ret;
    char *tokens[MAX_TOKENS + 1];
    DEB(printf("gethostbyaddr\n"));
    if((file = fopen(hfname, "r")) == NULL)
        return NULL;
    if((buf = malloc(BUF_LEN)) == NULL)
    {
        fclose(file);
        return NULL;
    }
    memcpy(find, addr, len);
    find[len] = 0x00;
    memset(tokens, 0x00, ArraySize(tokens));
    for(i = 0; i < MAX_TOKENS; i++)
    {
        if((tokens[i] = malloc(MAXHOSTNAMELEN)) == NULL)
        {
            free_tokens(tokens, 0);
            fclose(file);
            free(buf);
            return NULL;
        }
    }
    while(fgets(buf, BUF_LEN, file) != NULL)
    {
       if((cp = strchr(buf, '#')) == buf)
           continue;
       if(cp != NULL)
           *cp = (char) 0x00;
       strtoupr(buf);

       for(i=0; i< MAX_TOKENS; i++)
           *tokens[i] = (char) 0x00;
       ret = sscanf(buf, "%s%s%s%s%s", tokens[0], tokens[1], tokens[2],
                                  tokens[3], tokens[4]);

        DEB(printf("token 0 = %s\n", tokens[0]));
        DEB(printf("token 1 = %s\n", tokens[1]));
        DEB(printf("token 2 = %s\n", tokens[2]));
        DEB(printf("token 3 = %s\n", tokens[3]));
        DEB(printf("token 4 = %s\n", tokens[4]));

        if(memcmp(tokens[0], find, len) == 0)
        {
                hp = malloc(sizeof(MY_HP));
                DEB(printf("Found %s\n", tokens[0]));
                DEB(printf("Number of objects = %d\n", ret));
                hp->addrlist[0]     = tokens[0];         // Only one address
                hp->addrlist[1]     = NULL;              // Null terminated
                hp->hst.h_name      = tokens[1];         // Name of host
                hp->hst.h_aliases   = tokens;	         // Alias list
                hp->hst.h_addrtype  = AF_INET;           // Only one allowed
                hp->hst.h_addr_list = hp->addrlist;      // list of addresses 
                hp->hst.h_length    = len;               // length of address
                free_tokens(tokens, ret);
                tokens[ret] = NULL;
                fclose(file);
                free(buf);
                return &hp->hst; 
         }
    }
    free_tokens(tokens, 0);
    fclose(file);
    free(buf);
    return NULL;
}
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// The sethostent() function specifies, if stayopen is true (1), that
// a connected TCP socket should be used for the name server queries
// and that the connection should remain open during successive queries.
// Otherwise, name server queries will use UDP datagrams.
//
void sethostent(int stayopen)
{
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// The endhostent() function ends the use of a TCP connection for name
// server queries.
//
void endhostent()
{
}
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// The (obsolete) herror() function prints the error message associated
// with the current value of h_errno on stderr.
//
void herror(const char *s)
{
     fprintf(stderr,"Obsolete herror value is %d\n", h_errno);
}
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  The (obsolete) hstrerror() function takes an error number (typically
//  h_errno) and returns the corresponding message string.
//
const char *hstrerror(int err)
{
    return NULL;
}
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
