//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//
//  Copyright(c)  2005  Richard B. Johnson
//
//  This program may be distributed under the GNU Public License
//  version 2, as published by the Free Software Foundation, Inc.,
//  59 Temple Place, Suite 330 Boston, MA, 02111.
//
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <malloc.h>
#include <errno.h>
#include <stdarg.h>
#include <ctype.h>
#include <dirent.h>
#include <sys/stat.h>

extern void __set_errno(int);
extern int sys_stat(const char *fname, struct stat *stat);
extern int sys_lstat(const char *fname, struct stat *stat);
extern int sys_fstat(int fd, struct stat *stat);
extern int sys_getdents(int fd, struct dirent *dirp, size_t count);
extern size_t strnlen(const char *string, size_t len);
#define FAIL -1


struct _stat {
        unsigned short nsp_st_dev;
        unsigned short nsp__pad1;
        unsigned long  nsp_st_ino;
        unsigned short nsp_st_mode;
        unsigned short nsp_st_nlink;
        unsigned short nsp_st_uid;
        unsigned short nsp_st_gid;
        unsigned short nsp_st_rdev;
        unsigned short nsp__pad2;
        unsigned long  nsp_st_size;
        unsigned long  nsp_st_blksize;
        unsigned long  nsp_st_blocks;
        unsigned long  nsp_st_atime;
        unsigned long  nsp__unused1;
        unsigned long  nsp_st_mtime;
        unsigned long  nsp__unused2;
        unsigned long  nsp_st_ctime;
        unsigned long  nsp__unused3;
        unsigned long  nsp__unused4;
        unsigned long  nsp__unused5;
};

int __fxstat(int version, int fd, struct stat *stat)
{
    int ret;
    struct _stat sys;
    if((ret = sys_fstat(fd, (struct stat *) &sys)) == FAIL)
        return ret;
    stat->st_dev     = sys.nsp_st_dev;
    stat->st_ino     = sys.nsp_st_ino;
    stat->st_mode    = sys.nsp_st_mode;
    stat->st_nlink   = sys.nsp_st_nlink;
    stat->st_uid     = sys.nsp_st_uid;
    stat->st_gid     = sys.nsp_st_gid;
    stat->st_rdev    = sys.nsp_st_rdev;
    stat->st_size    = sys.nsp_st_size;
    stat->st_blksize = sys.nsp_st_blksize;
    stat->st_blocks  = sys.nsp_st_blocks;
    stat->st_atime   = sys.nsp_st_atime;
    stat->st_mtime   = sys.nsp_st_mtime;
    stat->st_ctime   = sys.nsp_st_ctime;
#if 0
    printf ("st_dev = %d\n", stat->st_dev);
    printf ("st_ino = %d\n", stat->st_ino);
    printf ("st_mode = %d\n",stat->st_mode);
    printf ("st_nlink = %d\n",stat->st_nlink );
    printf ("st_uid = %d\n", stat->st_uid);
    printf ("st_gid = %d\n", stat->st_gid);
    printf ("st_rdev = %d\n", stat->st_rdev);
    printf ("st_size = %d\n", stat->st_size);
    printf ("st_blocksize = %d\n", stat->st_blksize);
    printf ("st_blocks = %d\n", stat->st_blocks);
    printf ("st_atime = %08x\n", stat->st_atime);
    printf ("st_mtime = %08x\n", stat->st_mtime);
    printf ("st_ctime = %08x\n", stat->st_ctime);
#endif
    return ret;
}

int __lxstat(int version, const char *fname, struct stat *stat)
{
    int ret;
    struct _stat sys;
    if((ret = sys_lstat(fname, (struct stat *) &sys)) == FAIL)
        return ret;
    stat->st_dev     = sys.nsp_st_dev;
    stat->st_ino     = sys.nsp_st_ino;
    stat->st_mode    = sys.nsp_st_mode;
    stat->st_nlink   = sys.nsp_st_nlink;
    stat->st_uid     = sys.nsp_st_uid;
    stat->st_gid     = sys.nsp_st_gid;
    stat->st_rdev    = sys.nsp_st_rdev;
    stat->st_size    = sys.nsp_st_size;
    stat->st_blksize = sys.nsp_st_blksize;
    stat->st_blocks  = sys.nsp_st_blocks;
    stat->st_atime   = sys.nsp_st_atime;
    stat->st_mtime   = sys.nsp_st_mtime;
    stat->st_ctime   = sys.nsp_st_ctime;
#if 0
    printf ("st_dev = %d\n", stat->st_dev);
    printf ("st_ino = %d\n", stat->st_ino);
    printf ("st_mode = %d\n",stat->st_mode);
    printf ("st_nlink = %d\n",stat->st_nlink );
    printf ("st_uid = %d\n", stat->st_uid);
    printf ("st_gid = %d\n", stat->st_gid);
    printf ("st_rdev = %d\n", stat->st_rdev);
    printf ("st_size = %d\n", stat->st_size);
    printf ("st_blocksize = %d\n", stat->st_blksize);
    printf ("st_blocks = %d\n", stat->st_blocks);
    printf ("st_atime = %08x\n", stat->st_atime);
    printf ("st_mtime = %08x\n", stat->st_mtime);
    printf ("st_ctime = %08x\n", stat->st_ctime);
#endif
    return ret;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *  This performs xstat translation.
 */
int __xstat(int vers, const char *fname, struct stat *stat)
{
    int ret;
    struct _stat sys;
    if((ret = sys_stat(fname, (struct stat *) &sys)) == FAIL)
        return ret;
    stat->st_dev     = sys.nsp_st_dev;
    stat->st_ino     = sys.nsp_st_ino;
    stat->st_mode    = sys.nsp_st_mode;
    stat->st_nlink   = sys.nsp_st_nlink;
    stat->st_uid     = sys.nsp_st_uid;
    stat->st_gid     = sys.nsp_st_gid;
    stat->st_rdev    = sys.nsp_st_rdev;
    stat->st_size    = sys.nsp_st_size;
    stat->st_blksize = sys.nsp_st_blksize;
    stat->st_blocks  = sys.nsp_st_blocks;
    stat->st_atime   = sys.nsp_st_atime;
    stat->st_mtime   = sys.nsp_st_mtime;
    stat->st_ctime   = sys.nsp_st_ctime;
#if 0
    printf ("st_dev = %d\n", stat->st_dev);
    printf ("st_ino = %d\n", stat->st_ino);
    printf ("st_mode = %d\n",stat->st_mode);
    printf ("st_nlink = %d\n",stat->st_nlink );
    printf ("st_uid = %d\n", stat->st_uid);
    printf ("st_gid = %d\n", stat->st_gid);
    printf ("st_rdev = %d\n", stat->st_rdev);
    printf ("st_size = %d\n", stat->st_size);
    printf ("st_blocksize = %d\n", stat->st_blksize);
    printf ("st_blocks = %d\n", stat->st_blocks);
    printf ("st_atime = %08x\n", stat->st_atime);
    printf ("st_mtime = %08x\n", stat->st_mtime);
    printf ("st_ctime = %08x\n", stat->st_ctime);
#endif
    return ret;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/


