//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//
//  Copyright(c)  2005  Richard B. Johnson
//
//  This program may be distributed under the GNU Public License
//  version 2, as published by the Free Software Foundation, Inc.,
//  59 Temple Place, Suite 330 Boston, MA, 02111.
//
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


#include <stdlib.h>
#include <errno.h>
#include <sys/shm.h>
#include <syscall.h>
#include "ipc.h"

//
// Attach the shared memory segment associated with SHMID to the data
// segment of the calling process.  SHMADDR and SHMFLG determine how
// and where the segment is attached.
//
void * shmat (int shmid, const void *shmaddr, int shmflg)
{
    int retval;
    unsigned long raddr;

    retval = __ipc(IPCOP_shmat, shmid, shmflg, (int) &raddr, (void *) shmaddr);
    return ((unsigned long int) retval > -(unsigned long int) SHMLBA
            ? (void *) retval : (void *) raddr);
}

int shmctl (int shmid, int cmd, struct shmid_ds *buf)
{
    return __ipc(IPCOP_shmctl, shmid, cmd, 0, buf);
}
//
// Detach shared memory segment starting at address specified by SHMADDR
// from the caller's data segment.
//
int shmdt (const void *shmaddr)
{
    return __ipc(IPCOP_shmdt, 0, 0, 0, (void *) shmaddr);
}
//
//  Return an identifier for an shared memory segment of at least size SIZE
//  which is associated with KEY.
//
int shmget (key_t key, size_t size, int shmflg)
{
    return __ipc(IPCOP_shmget, key, size, shmflg, NULL);
}


