//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//
//  Copyright(c)  2005  Richard B. Johnson
//
//  This program may be distributed under the GNU Public License
//  version 2, as published by the Free Software Foundation, Inc.,
//  59 Temple Place, Suite 330 Boston, MA, 02111.
//
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

/*
 *   This impliments most all the termios stuff listed in
 *   the `man` pages.
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#define FAIL -1

extern void __set_errno(int);

int tcgetattr (int fd, struct termios *term)
{
    return ioctl(fd, TCGETS, term);
}
int tcsetattr (int fd, int actions, const struct termios *term)
{
    int act;
    switch(actions)
    {
    case TCSANOW:
        act = TCSETS;
        break;
    case TCSADRAIN:
        act = TCSETSW;
        break;
    case TCSAFLUSH:
        act = TCSETSF;
        break;
    default:
        act = TCSETA;
    }
    return ioctl(fd, act, term);
}

int tcsendbreak(int fd, int duration)
{
    return ioctl(fd, TCSBRK, duration);
}

int tcdrain(int fd)
{
    return ioctl(fd, TCFLSH, TCIOFLUSH);
}

int tcflush (int fd, int queue_selector )
{
    return ioctl(fd, TCFLSH, queue_selector);
}

int tcflow (int fd, int action)
{
    return ioctl(fd, TCXONC, action);
}

void cfmakeraw (struct termios *term )
{
    term->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
    term->c_oflag &= ~OPOST;
    term->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
    term->c_cflag &= ~(CSIZE|PARENB);
    term->c_cflag |= CS8;
}

speed_t cfgetospeed (const struct termios *termios_p)
{
    return 0;
}

int cfsetospeed (struct termios *termios, speed_t speed)
{
    termios->c_cflag &= ~CBAUD;
    termios->c_cflag |= speed;
    return 0;
}

speed_t cfgetispeed (const struct termios *termios)
{
    return termios->c_cflag & CBAUD;
}

int cfsetispeed (struct termios *termios, speed_t speed)
{
    return cfsetospeed(termios, speed);
}

pid_t tcgetpgrp(int fd)
{
    pid_t grp;
    if(ioctl(fd, TIOCGPGRP, &grp)<0)
        perror("ioctl(TIOCGPGRP)");
    return grp;
}

int tcsetpgrp (int fd, pid_t pgrpid)
{
    return ioctl(fd, TIOCSPGRP, &pgrpid);
}
//
//   This returns TRUE if the fd is connected with a terminal
//   and FALSE if not.
//
int isatty(int fd)
{
    struct stat sb;
    if(fstat(fd, &sb) == FAIL)
        return 0;
    return(int) (S_IFCHR & sb.st_mode);
}

