//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//
//  Copyright(c)  2005  Richard B. Johnson
//
//  This program may be distributed under the GNU Public License
//  version 2, as published by the Free Software Foundation, Inc.,
//  59 Temple Place, Suite 330 Boston, MA, 02111.
//
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *   File dir.c                  Created 12-JUL-2002    Richard B. Johnson
 *
 *   This obtains the complete directory of every file on the Platinum
 *   Control Computer.
 */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>
#include <time.h>
#include <signal.h>
#define LINES 15
#define TRUE 1
#define FALSE 0

#undef readdir

int ret2quit()
{
    return 0;
}

typedef struct {
    mode_t  mask;
    char    token;
    } MODE;
static const char cwd[]=".";
static const char par[]="..";
static const char prc[]="proc";
static const char sls = '/';
static int pdir(const char *, long long *total, long *files, int *quit );
int main(void);

static MODE perms[]= {
                     { S_IRUSR,  'r' },
                     { S_IWUSR,  'w' },
                     { S_IXUSR,  'x' },
                     { S_IRGRP,  'r' },
                     { S_IWGRP,  'w' },
                     { S_IXGRP,  'x' },
                     { S_IROTH,  'r' },
                     { S_IWOTH,  'w' },
                     { S_IXOTH,  'x' }};

static MODE type[]= {
                    { S_IFSOCK, 's'  },
                    { S_IFLNK, 'l'   },
                    { S_IFREG, '-'   },
                    { S_IFBLK, 'b'   },
                    { S_IFDIR, 'd'   },
                    { S_IFCHR, 'c'   },
                    { S_IFIFO, 'f'   }};


#define ArraySize(a)  (sizeof(a) / sizeof(a[0]))

static int pdir(const char *dent, long long *total, long *files, int *quit)
{
    struct stat sb;
    char *buf, *ins;
    struct dirent *d;
    DIR *dir;
    int nopar, nocwd;
    size_t ins_len, buf_len, i;

    if(*quit) return TRUE;
    i = strlen(dent);
    ins_len = (i + 3) & ~1;
    if((ins = malloc(ins_len)) == NULL)
        return TRUE;
    memcpy(ins, dent, i--);
    if(ins[i++] != sls)
        ins[i++] = sls;
    ins[i] = 0x00;

    if((dir = opendir(ins)) != NULL)
    {
        while ((d = readdir(dir)) != NULL)
        {
            buf_len = (ins_len + strlen(d->d_name) + 2) & ~1;
            if((buf = malloc(buf_len)) == NULL)
                break;
            strcat(strcpy(buf, ins), d->d_name);
            if(lstat(buf, &sb) == -1)
            {                                           /* Was just deleted */
                free(buf);
                continue;
            }
            nopar = memcmp(d->d_name, par, sizeof(par));
            nocwd = memcmp(d->d_name, cwd, sizeof(cwd));
            if((S_ISDIR(sb.st_mode))                 &&  /* If a directory   */
               (memcmp(d->d_name, prc, sizeof(prc))) &&  /* Not the proc dir */
               (nocwd)                               &&  /* Not cwd token    */
               (nopar)                             )     /* Not parent token */
            {
                if(pdir(buf, total, files, quit))        /* Yes, recursion   */
                {
                    (void)closedir(dir);
                    free(buf);
                    free(ins);
                    return TRUE;
                }
            }
            else
            {
                char mode[0x10], ltim[0x20], *cp;
                *mode = 0x00;
                if(nopar)
                {
                    if(!nocwd)
                       buf[strlen(buf) - 1] = 0x00;
                    strncpy(ltim, asctime(localtime(&sb.st_mtime)),sizeof(ltim));
                    ltim[strlen(ltim)-1]= 0x00;
                    for(i = 0, cp = mode; i< ArraySize(type); i++)
                        if((sb.st_mode & type[i].mask) == type[i].mask)
                            break;
                    *cp++ = type[i].token;
                    memset(cp, '-', ArraySize(perms));
                    for(i = 0; i< ArraySize(perms); i++)
                        if(sb.st_mode & perms[i].mask)
                            cp[i] = perms[i].token;
                    cp[i] = 0x00;
                    printf("%s %-4d %-4d %-4d %-9lu %s %s\n",
                                      mode, sb.st_nlink, sb.st_uid,
                                      sb.st_gid, sb.st_size, &ltim[3], buf);
                    fflush(stdout);
                    (*total) += (long long) sb.st_size;
                    if(!(++(*files) % LINES))
                    {
                        if(ret2quit())
                        {
                            (void)closedir(dir);
                            free(buf);
                            free(ins);
                            return (*quit = TRUE);
                        }
                    }
                }
            }
            free(buf);
        }
        (void)closedir(dir);
    }
    free(ins);
    return FALSE;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

int main()
{
    long long total;
    long files;
    int quit;
    total = 0LL;
    files = 0L;
    quit  = 0;
    if(pdir("/", &total, &files, &quit))
        fprintf(stderr, "Aborted\n");
    fprintf(stdout, "Total %lld bytes in %ld files\n", total, files);
    fflush(stdout);
    return 0;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=- E N D -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
